package edu.usc.ict.gordon.copa.eval;

import edu.usc.ict.gordon.util.UnsupportedDataFormatException;

import static java.lang.System.out;

/**
 * Data structure for managing an item response
 *
 * @author Cosmin Adrian Bejan
 * @version 1.0
 * @since JDK1.6 | August 2010
 */
public class ResponseItem {
  /* Response id */
  private int id;
  /* A binary array for encoding a gold or system response where each cell is associated with
     a category. There is only one cell that has a value of 1 (the other cells are set to 0) 
     and the category associated with this cell represents the response for this item. */
  private int[] annotations;
  
  /**
   * Creates a <tt>ResponseItem</tt> instance from a string representation.
   *
   * @param format the string format encoding a <tt>ResponseItem</tt> 
   */
  public ResponseItem(String format, int K) {
    String[] toks = format.split("\\s+");
    if(toks.length != K + 1)
      throw new UnsupportedDataFormatException("Not a valid format: ["+format+"]");
    
    id = Integer.parseInt(toks[0]);
    annotations = new int[K];
    for(int i = 0; i < K; i++) 
      annotations[i] = Integer.parseInt(toks[i+1]);
    checkValidItem();
  }
  
  /**
   * Verifies if the item has a proper format: only one 1-cell and the rest 0-cells.
   *
   */
  private void checkValidItem() {
    boolean unitFlag = false;
    for(int i=0; i < annotations.length; i++) {
      if(annotations[i] == 1) {
        if(unitFlag)
          throw new UnsupportedDataFormatException("Multiple 1 cells");
        else 
          unitFlag = true;
      } else {
        if(annotations[i] != 0)
          throw new UnsupportedDataFormatException("Cells different from 0 and 1");
      }        
    }
  }
  
  /**
   * Returns the id of this response item
   *
   */
  public int getID() {
    return id;
  }
  
  /**
   * Returns the category selected for this item
   *
   */
  public int getSelectedCategory() {
    for(int i=0; i < annotations.length; i++)
      if(annotations[i] == 1) return i;
    throw new UnsupportedDataFormatException("Wrong single item annotations");
  }
  
  /**
   * Prints the representation for this response item
   *
   */
  public void print() {
    out.printf("single-item[%3d]: ",id);
    for(int i=0; i < annotations.length; i++) 
      out.print("   "+annotations[i]);
    out.println();
  }
}
