package edu.usc.ict.gordon.util;

import java.io.Serializable;

import static java.lang.System.out;

/**
 * Data structure for representing a (double, double) pair. 
 *
 * @author Cosmin Adrian Bejan
 * @version 1.0
 * @since JDK1.5 | June 2004
 */
public class DoublePair implements Serializable {
  private static final long serialVersionUID = 123L;
  protected double first;
  protected double second;  

  /**
   * Constructs a <tt>DoublePair</tt> object from another <tt>DoublePair</tt> object
   *
   */
  public DoublePair(double f, double s) {
    setPair(f,s);
  }

  /**
   * Constructs an DoublePair object from two double values
   *
   */
  public DoublePair(DoublePair pair) {
    setPair(pair.first, pair.second);
  }

  /**
   * Creates a new copy of this object
   *
   */
  public DoublePair clone() {
    DoublePair p = new DoublePair(this);
    return p;
  }

  /**
   * Sets the values
   * 
   * @param f the new value for the first double
   * @param s the new value for the second double
   */
  public void setPair(double f, double s) {
    first = f;
    second = s;
  }

  /**
   * Sets the first value
   *
   */
  public void setFirst(double f) {
    first = f;
  }
  
  /**
   * Accessor for the first value
   *
   */
  public double getFirst() {
    return first;
  }
  
  /**
   * Sets the second value
   *
   */
  public void setSecond(double s) {
    second = s;
  }
  
  /**
   * Accessor for the second value
   *
   */
  public double getSecond() {
    return second;
  }

  /**
   * Implementation of the <code>equals</code> method
   *
   */
  @Override
  public boolean equals(Object that) {
    if ( this == that ) return true;
    if ( !(that instanceof DoublePair) ) return false;
    DoublePair thatPair = (DoublePair)that;
    return ((first == thatPair.first) && (second == thatPair.second));
  }

  /**
   * Implementation of the <code>hasCode</code> method.
   *
   */
  @Override
  public int hashCode() {
    int f = (int)Double.doubleToLongBits(first);
    int s = (int)Double.doubleToLongBits(second);
    return ((f + s) * (f + s) + 3 * f + s) / 2 ;
  }   

  /**
   * Implementation of the <code>toString</code> method
   *
   */
  @Override
  public String toString() {
    return "d2["+first+"|"+second+"]";
  }

  /**
   * Converts this object to s string format representation
   * 
   * @return a <tt>String</tt> in the first:second format
   */
  public String getDotsFormat() {
    return ""+first+":"+second;
  }
 
  /**
   * Prints to System.out a string format of this object
   *
   */
  public void println() {
    out.println(first+" - "+second);
  } 
  
  /**
   * Prints to System.out a string format of this object
   *
   */
  public void print() {
    out.print(first+" - "+second);
  }

  /**
   * Test whether this <tt>DoublePair</tt> object is a point (i.e., its values are equal).
   *
   */
  public boolean isPoint() {
    return first == second;
  }

  /**
   * Test whether this <tt>DoublePair</tt> object has its values in the increasing order
   *
   */
  public boolean isIncreasing() {
    return first <= second;
  }

  /**
   * Test whether this <tt>DoublePair</tt> object has its values in the strictly increasing order
   *
   */
  public boolean isStrictlyIncreasing() {
    return first < second;
  }

  /**
   * Swaps the values of this <tt>DoublePair</tt> object
   *
   */
  public void swap() {
    double aux;
    aux = first;
    first = second;
    second = aux;
  }
}
