package edu.usc.ict.gordon.util;

import static java.lang.System.out;

/**
 * Data structure for representing an (int, int) pair. 
 *
 * @author Cosmin Adrian Bejan
 * @version 1.0
 * @since JDK1.5 | February 2004
 */
public class IntPair implements java.io.Serializable {
  private static final long serialVersionUID = 97575822457L;
  
  protected int first;
  protected int second;  

  /**
   * Constructs an <tt>IntPair</tt> object from another <tt>IntPair</tt> object
   *
   */
  public IntPair(int f, int s) {
    setPair(f,s);
  }

  /**
   * Constructs an IntPair object from two int values
   *
   */
  public IntPair(IntPair pair) {
    setPair(pair.first, pair.second);
  }

  /**
   * Sets the values
   * 
   * @param f the new value for the first int
   * @param s the new value for the second int
   */
  public void setPair(int f, int s) {
    first = f;
    second = s;
  }

  /**
   * Sets the first value
   * 
   */
  public void setFirst(int f) {
    first = f;
  }
  
  /**
   * Accessor for the first value
   * 
   */
  public int getFirst() {
    return first;
  }
  
  /**
   * Sets the second value
   * 
   */
  public void setSecond(int s) {
    second = s;
  }
  
  /**
   * Accessor for the second value
   * 
   */
  public int getSecond() {
    return second;
  }

  /**
   * Increments the first value by 1
   * 
   */
  public void incFirst() {
    incFirst(1);
  }

  /**
   * Increments the first value
   * 
   * @param val the increment value
   */
  public void incFirst(int val) {
    first += val;
  }

  /**
   * Increments the second value by 1
   * 
   */
  public void incSecond() {
    incSecond(1);
  }
  
  /**
   * Increments the second value
   * 
   * @param val the increment value
   */
  public void incSecond(int val) {
    second += val;
  }

  /**
   * Increments the two pair values
   * 
   * @param pair increment for the pair values
   */
  public void inc(IntPair pair) {
    inc(pair.first, pair.second);
  }

  /**
   * Increments the two pair values. 
   * 
   * @param f increment for the first value
   * @param s increment for the second value
   */
  public void inc(int f, int s) {
    incFirst(f);
    incSecond(s);
  }

  /**
   * Implementation of the <code>equals</code> method
   *
   */
  @Override
    public boolean equals(Object that) {
    if ( this == that ) return true;
    if ( !(that instanceof IntPair) ) return false;
    IntPair thatPair = (IntPair)that;
    return ((first == thatPair.first) && (second == thatPair.second));
  }

  /**
   * Tests whether the two values are equal
   */
  public boolean equals() {
    return (first == second);
  }

  /**
   * Implementation of the <code>hasCode</code> method
   *
   */
  @Override
    public int hashCode() {
    return ((first+second)*(first+second) + 3*first + second)/2 ;
  }  

  /**
   * Implementation of the <code>toString</code> method
   *
   */
  @Override
    public String toString() {
    return "i2["+first+"|"+second+"]";
  }

  /**
   * Converts this object to s string format representation
   * 
   * @return a <tt>String</tt> in the first:second format
   */
  public String getDotsFormat() {
    return ""+first+":"+second;
  }

  /**
   * Prints to System.out a string format of this object
   *
   */
  public void println() {
    out.println(first+" - "+second);
  } 
  
  /**
   * Prints to System.out a string format of this object
   *
   */
  public void print() {
    out.print(first+" - "+second);
  }
}
